@@ -124,6 +124,9 @@ class JAAQLModel(BaseJAAQLModel):
         execute_supplied_statement(self.jaaql_lookup_connection, QUERY__drop_email_account, {KEY__name: name, KEY__tenant: tenant})
         self.email_manager.reload_service()
 
+    def create_database(self, connection: DBInterface, name: str):
+        execute_supplied_statement(connection, "SELECT create_tenant_database(:name)", {KEY__name: name})
+
     def drop_database(self, tenant: str, connection: DBInterface, name: str, delete_record: bool, silent: bool):
         self.check_admin(connection, "drop_database")
         if delete_record is None:
